@@ -48,7 +48,7 @@ static bool floattest()
 {
 #if __ppc__ || __ppc64__
 	return dofloattest(1.0,2.0,3.0,4.0,5.0,6.0,7.0,8.0,9.0,10.0,11.0,12.0,13.0);
-#elif __i386__
+#elif __i386__ || __x86_64__ 
 	return true;
 #else
 	#error unknown architecture
